@@ -1,5 +1,6 @@
 import spacy
 from spacy.pipeline import EntityRuler
+from spacy.language import Language
 import time
 from typing import List
 from hc_nlp import constants, logging
@@ -7,8 +8,16 @@ from hc_nlp import constants, logging
 logger = logging.get_logger(__name__)
 
 
-class ThesaurusMatcher:
+@Language.factory(
+    "thesaurus_matcher",
+    default_config={"overwrite_ents": False, "case_sensitive": False},
+)
+def thesaurus_matcher(
+    nlp, name, thesaurus_path: str, case_sensitive: bool, overwrite_ents: bool
+):
     """
+    Factory function for a ThesaurusMatcher.
+
     The ThesaurusMatcher lets you add spans to `Doc.ents` using exact phrase
     matches from an imported phrasebook (Thesaurus). It can be combined with 
     the statistical `EntityRecognizer` to boost accuracy, or used on its own 
@@ -17,61 +26,30 @@ class ThesaurusMatcher:
     `nlp.add_pipe`.
     """
 
-    def __init__(
-        self,
-        nlp,
-        thesaurus_path: str,
-        case_sensitive: bool,
-        overwrite_ents: bool = False,
-    ):
-        """
-        Initialise the ThesaurusMatcher. `thesaurus_path` must point to a .jsonl
-        file with each line in the following format (with the `id` key optional):
+    logger.info(f"Loading thesaurus from {thesaurus_path}")
+    other_pipes = [p for p in nlp.pipe_names if p != "tagger"]
 
-        ```
-        {
-            "label": "<entity label>",
-            "pattern": "<text to match>",
-            "id": "<ID which the above text unambiguously refers to. Optional.>",
-        }
-        ```
+    start = time.time()
 
-        Args:
-            nlp: spacy model
-            thesaurus_path (str): path to the thesaurus
-            case_sensitive (bool): [description]
-        """
-        if case_sensitive:
-            self.ruler = EntityRuler(nlp)
-        else:
-            self.ruler = EntityRuler(nlp, phrase_matcher_attr="LOWER")
+    # set config for new entityruler object
+    if case_sensitive:
+        with nlp.disable_pipes(*other_pipes):
+            ruler = EntityRuler(nlp, overwrite_ents=overwrite_ents).from_disk(
+                thesaurus_path
+            )
+    else:
+        with nlp.disable_pipes(*other_pipes):
+            ruler = EntityRuler(
+                nlp, overwrite_ents=overwrite_ents, phrase_matcher_attr="LOWER"
+            ).from_disk(thesaurus_path)
 
-        self.nlp = nlp
-        self.thesaurus_path = thesaurus_path
-        self._add_thesaurus_to_ruler()
-        self.ruler.overwrite = overwrite_ents
+    end = time.time()
+    logger.info(f"{len(ruler)} term thesaurus imported in {int(end-start)}s")
 
-    def _add_thesaurus_to_ruler(self):
-        """
-        Load thesaurus from disk and add to self.ruler
-        """
-        logger.info(f"Loading thesaurus from {self.thesaurus_path}")
-        other_pipes = [p for p in self.nlp.pipe_names if p != "tagger"]
-
-        start = time.time()
-        with self.nlp.disable_pipes(*other_pipes):
-            self.ruler.from_disk(self.thesaurus_path)
-
-        end = time.time()
-        logger.info(f"{len(self.ruler)} term thesaurus imported in {int(end-start)}s")
-
-    def __call__(self, doc: spacy.tokens.Doc) -> spacy.tokens.Doc:
-        """
-        Inherits from EntityRuler behaviour.
-        """
-        return self.ruler(doc)
+    return ruler
 
 
+@Language.factory("entity_filter")
 class EntityFilter:
     """
     The EntityFilter filters out any entities in `Doc.ents` that aren't likely to be
@@ -87,6 +65,8 @@ class EntityFilter:
 
     def __init__(
         self,
+        nlp,
+        name: str,
         max_token_length: int = 1,
         remove_all_lower: bool = True,
         remove_all_upper: bool = True,
@@ -105,6 +85,7 @@ class EntityFilter:
             ent_labels_ignore (List[str], optional): Entities with labels to ignore 
                 when making the corrections.
         """
+        self.nlp = nlp
         self.max_token_length = max_token_length
         self.remove_all_lower = remove_all_lower
         self.remove_all_upper = remove_all_upper
@@ -149,13 +130,31 @@ class EntityFilter:
         return doc
 
 
+def pattern_matcher(nlp, name: str, patterns: List[dict]):
+    """
+    Create an EntityRuler object loaded with a list of patterns.
+
+    Args:
+        nlp : Spacy model
+        patterns (List[dict]): for the EntityRuler. See https://spacy.io/usage/rule-based-matching#entityruler 
+
+    Returns:
+        Spacy EntityRuler component
+    """
+    ruler = EntityRuler(nlp)
+    ruler.add_patterns(patterns)
+
+    return ruler
+
+
+@Language.factory("pattern_matcher")
 class PatternMatcher:
     """
     An EntityRuler object initiated with a pattern. Used for built-in `hc_nlp`
     matchers.
     """
 
-    def __init__(self, nlp, patterns: List[dict]):
+    def __init__(self, nlp, name: str, patterns: List[dict]):
         """
         Initialise the PatternMatcher.
 
@@ -173,9 +172,13 @@ class PatternMatcher:
         return self.ruler(doc)
 
 
+@Language.factory("date_matcher")
 class DateMatcher(PatternMatcher):
-    def __init__(self, nlp):
-        super().__init__(nlp, constants.DATE_PATTERNS)
+    def __init__(self, nlp, name):
+        # TODO: inherit from pattern_matcher
+        super().__init__(nlp, name, constants.DATE_PATTERNS)
+        # self.ruler = EntityRuler(nlp)
+        # self.ruler.add_patterns(constants.DATE_PATTERNS)
 
     def _add_centuries_to_doc(self, doc: spacy.tokens.Doc) -> spacy.tokens.Doc:
         """
@@ -183,10 +186,8 @@ class DateMatcher(PatternMatcher):
         'century' or 'centuries', checking that the previous word is an ordinal, and then returning all 
         the immediate children of the token 'century'. It then checks for occurrences of "nth (and/or/to) 
         mth centuries", as well as "AD" or "BC" after the word century/centuries.
-
         Args:
             doc (spacy.tokens.Doc)
-
         Returns:
             spacy.tokens.Doc
         """
@@ -200,7 +201,7 @@ class DateMatcher(PatternMatcher):
                 ):
                     try:
                         first_child = next(token.children)
-                    except Exception:  #  noqa: E722
+                    except Exception:  # noqa: E722
                         # if the token has no children, use the ordinal token as first_child
                         first_child = doc[idx - 1]
 
@@ -227,16 +228,21 @@ class DateMatcher(PatternMatcher):
                             end += 1
 
                     date_entity = spacy.tokens.Span(doc, start, end, label="DATE")
-                    doc.ents = list(doc.ents) + [date_entity]
+                    try:
+                        doc.ents = list(doc.ents) + [date_entity]
+                    except Exception:
+                        # TODO: check for overlap instead of just failing
+                        # TODO: handle the specific spaCy error
+                        logger.warn(
+                            f"Failed to add DATE entity {date_entity.text} in pos {(start, end)} to text {doc.text}"
+                        )
         return doc
 
     def __call__(self, doc: spacy.tokens.Doc) -> spacy.tokens.Doc:
         """
         Detects centuries then patterns from `constants.DATE_PATTERNS`.
-
         Args:
             doc (spacy.tokens.Doc)
-
         Returns:
             spacy.tokens.Doc
         """
@@ -246,19 +252,11 @@ class DateMatcher(PatternMatcher):
         return doc
 
 
+@Language.factory("map_entity_types")
 class MapEntityTypes:
     def __init__(
-        self,
-        nlp,
-        mapping: dict = constants.SPACY_TO_HC_ENTITY_MAPPING,
-        validate_mapping: bool = True,
+        self, nlp, name: str, mapping: dict = constants.SPACY_TO_HC_ENTITY_MAPPING,
     ):
-        entities_missing_from_mapping = set(nlp.entity.labels) - set(mapping.keys())
-
-        if validate_mapping and (len(entities_missing_from_mapping) > 0):
-            logger.warning(
-                f"The following entity labels from the Spacy model are not in the provided mapping: {', '.join(list(entities_missing_from_mapping))}. They will not be changed."
-            )
 
         self.mapping = mapping
         self.nlp = nlp
